import click
import yaml
import json
from tabulate import tabulate
from abak_config import set_configuration_key
from abak_shared_functions import get_clean_config, Sorry
from iterfzf import iterfzf
from os import environ

headers = ['Project', 'Client', 'Price']

@click.group()
def context():
    """
    Context operations for Abak. Select which project id and client id to use for the commands.
    """
    pass

@click.command(name='list')
@click.option('--output', '-o', help="Type of output to show the contexts", type=click.Choice(['table', 'json', 'yaml', 'yml']), default='table')
def context_list(output):
    '''
    Lists the available contexts
    '''
    contexts = get_contexts()
    if len(contexts) == 0:
        raise Sorry("there are no contexts created")
    if output == 'table':
        rows = [[context]+[contexts[context].get(header, 0)for header in headers]for context in contexts]
        click.echo(tabulate(rows, headers=['Context'] + headers))
    elif output in ['yaml', 'yml']:
        click.echo(yaml.dump(contexts))
    elif output == 'json':
        click.echo(json.dumps(contexts))

    click.echo()
    pass

@click.command(name='show')
@click.option('--name', '-n', help="Name of the context to show")
def context_show(name):
    '''
    shows the current context or the selected context.
    '''
    clean_config = get_clean_config()
    if not name:
        name = clean_config.get("current_context", None)
    
    if not name:
        raise Sorry('there are no contexts selected')

    contexts = clean_config.get('contexts', {})

    click.echo(f'{name}: {contexts.get(name, {})}')
    pass

@click.command(name='set')
@click.option('--name', '-n', help="Name of the context to be created")
@click.option('--client-id', '-c', help="ID of the client to be used in this context")
@click.option('--project-id', '-p', help="ID of the project to be used in this context")
@click.option('--price', '-r', help="ID of the project to be used in this context", default='0')
def context_set(name, client_id, project_id, price):
    '''
    Creates a new context
    '''
    contexts = get_contexts()
    contexts[name] = {'Project': project_id, 
                      'Client': client_id,
                      'Price': float(price)}
    set_configuration_key(contexts, 'contexts')
    pass

@click.command(name='select')
@click.option('--name', '-n', help='Name of the context to select')
def context_select(name):
    '''
    Selects a context
    '''
    contexts = get_contexts()
    if not name:
        selected_context = iterfzf([context for context in contexts], multi=False)
    else:
        if name not in contexts.keys():
            raise Sorry(f'context {name} does not exist')
        else:
            selected_context = name

    if selected_context:
        if isinstance(selected_context, list):
            set_configuration_key(selected_context[0], 'current_context')
        else:
            set_configuration_key(selected_context, 'current_context')
    pass

@click.command(name='remove')
@click.option('--name', '-n', help='Name of the context to be selected', required=True, prompt=True)
def context_remove(name):
    '''
    Removes a context
    '''
    config = get_clean_config()
    if name in config.get('contexts',{}).keys():
        del config['contexts'][name]
    set_configuration_key(config['contexts'], 'contexts')
    if 'current_context' in config.keys():
        if config.get('current_context') == name:
            set_configuration_key(None, 'current_context')

@click.group(name="price")
@click.pass_context
def context_price(ctx):
    '''
    Command group for managing Prices for Contexts
    '''
    pass

@click.command(name='set')
@click.option('--context', '-c', help="Context to set the price of the hour", default=lambda: environ.get('current_context', None))
@click.option('--price', help="Price of the hour in CAD", prompt="Price of the hour", required=True)
@click.pass_context
def context_price_set(ctx, context, price):
    '''
    Sets the price of the hour in a context
    '''
    if not context:
        raise Sorry(f"context cannot be empty")

    contexts = get_contexts()
    selected_context = contexts.get(context, None)

    if not selected_context:
        raise Sorry(f"context {context} not found")
    
    try:
        selected_context['Price'] = float(price)
    except:
        raise Sorry(f"can't convert {price} to a float number")

    contexts[context] = selected_context
    print(contexts)
    set_configuration_key(contexts, 'contexts')




@click.command(name='current')
def context_current():
    '''
    Shows the current context selected
    '''
    click.echo(f'current_context: {get_clean_config().get("current_context", None)}')

def get_contexts():
    return get_clean_config().get('contexts', {})


context_price.add_command(context_price_set)

context.add_command(context_list)
context.add_command(context_show)
context.add_command(context_set)
context.add_command(context_current)
context.add_command(context_select)
context.add_command(context_remove)
context.add_command(context_price)


from abak_config.set import set_configuration_key
import click
from iterfzf import iterfzf
from tabulate import tabulate
import json
from abak_shared_functions import get_config, httprequest
from os import environ

@click.group()
@click.pass_context
def project(ctx):
    '''
    Find projects to assign timesheet entries
    '''
    pass

@click.command(name='list')
@click.option('-o', '--output', help="The format of the output of this command", type=click.Choice(['json', 'table']), default='table')
@click.option('--client-id', '-c', help="ID of the client to search for projects.", default=lambda: environ.get('client_id', None))
@click.option('--all-projects', help='Ignore default selected client', is_flag=True)
@click.option('--query-text', '-q', help="Text to search for in the project name", default="")
@click.pass_context
def list_projects(ctx, output, client_id, all_projects, query_text):
    '''
    Lists the projects available for the given client
    '''
    get_projects(client_id if not all_projects else None, query_text, output)

@click.command(name='select')
@click.option('--client-id', '-c', help="ID of the client to search for projects", default=lambda: environ.get('client_id', None))
@click.option('--all-projects', help='Ignore default selected client', is_flag=True)
@click.pass_context
def project_select(ctx, client_id, all_projects):
    '''
    Selects the project to use as default
    '''
    projects_list = get_projects(client_id if not all_projects else None, "", 'python')
    if len(projects_list) > 1:
        selected_dirty = iterfzf([" - ".join([item.get('Id'), item.get('Display')]) for item in projects_list])
        selected = selected_dirty.split(' - ')[0]
    else:
        selected = projects_list[0].get('Id')
    set_configuration_key(selected, 'project_id')
    click.echo("project " + selected + " selected as default!")

def get_projects(client_id, query_text, output):
    config = get_config()
    body = {
        "isInModif": "false",
        "queryText": query_text,
        "start": 0,
        "limit": 22,
        "employeeId": config['user_id']
    }

    if client_id:
        body['clientId'] = client_id
    projects = httprequest('get', body, "/Abak/Common/GetTimesheetProjectsForPaginatedCombo")
    
    if output == "table": 
        headers = ["Id", "Display"]
        table = tabulate([[project[key]for key in project if key in headers]for project in projects.get('data')], headers=headers)
        print(table)
    elif output == 'json':
        print(json.dumps(projects.get('data')))
    elif output == 'python':
        return projects.get('data')

project.add_command(list_projects)
project.add_command(project_select) 


import click
from abak_shared_functions import get_config, write_config_file

@click.command(name='set')
@click.option('--key', '-k', help="The key to be set in this command", required=True, prompt=True)
@click.option('--value', '-v', help="The value for the key to be used", required=True, prompt=True)
@click.pass_context
def set_config(ctx, key, value):
    '''
    Sets the key:value for a configuration to be used in all the commands

    Args:
        key (str): The key to be set
        value (str): The value for the key
    '''
    set_configuration_key(value, key)
    
def set_configuration_key(value, key):
    configuration = get_config()
    configuration[key] = value
    write_config_file(configuration['config_file_path'], configuration)



from abak_shared_functions.requests_functions import httprequest
import click
import re
import json
import yaml
from tabulate import tabulate

from datetime import datetime, timedelta
import calendar
from os import environ

from abak_shared_functions import (
    Sorry, 
    get_config, 
    option_not_none, 
    generate_bs,
    get_date_format_from_abak
)

from abak_context import get_contexts
from project import get_projects

@click.group()
@click.pass_context
def timesheet(ctx):
    '''
    Commands to manage timesheet entries
    '''
    pass

def validate_entry_date(ctx, param, value):
    if not value:
        return None
    config = get_config()
    if value.upper() in ["MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY", "SUNDAY"]:
        return value
    try:
        datetime.strptime(value, config['date_format'])
        return value
    except:
        raise Sorry(f"date needs to be in the format {config['date_format']}")

def validate_description(ctx, param, value):
    if not value:
        raise Sorry("description ('--description', '-d') is a required parameter")
    elif len(value) <= 100:
        return value
    else:
        raise Sorry("description of the timesheet entry must not be larger than 100 characters")

@click.command(name='list')
@click.pass_context
@click.option('-d', '--date', help='The reference date to use for the query. Format in the config key "date_format"', required=False, callback=validate_entry_date)
@click.option('-o', '--output', help='The output type you want', type=click.Choice(['json', 'table', 'wide']), default='table')
@click.option('--query-range', '-r', help="The range to query for", type=click.Choice(['Weekly', 'Monthly', "Daily"]), default="Weekly")
@click.option('--show-totals', help="Show a summary of all projects for the given range", is_flag=True)
@click.option('--show-id', help="Shows the ID of each timesheeet entry", is_flag=True)
@click.option('--previous', help="Shows the previous iteration from the range", is_flag=True)
@click.argument('context-filter', required=False)
def timesheet_list(ctx, date, output, query_range, show_totals, show_id, previous, context_filter):
    '''
    Lists the timesheet entries in ABAK
    '''
    list_timesheet_entries(date, output, query_range, show_totals, show_id, previous, context_filter)
    
def list_timesheet_entries(date, output, query_range, show_totals, show_id, previous, context_filter):
    config = get_config()

    if not date:
        date = datetime.strftime(datetime.now(), format=config['date_format'])

    headers = {
        "Content-Type": "application/x-www-form-urlencoded"
    }
    if previous:
        #click.echo(date)
        date_datetime = datetime.strptime(date, config['date_format']) 
        delta = timedelta(days=date_datetime.day + 1) if query_range == "Monthly" else timedelta(days=1) if query_range == "Daily" else timedelta(weeks=1)
        date = datetime.strftime(date_datetime - delta, format=config['date_format']) 
        #click.echo(date)
    body = {
        "groupBy": "TransactType",
        "groupDir": "ASC",
        "summaryFields": "Quantity",
        "summaryTypes": "sum",
        "sort": "Date",
        "dir": "ASC",
        "employe": config['user_id'],
        "date": date + "T00:00:00",
        "range": query_range
    }

    output_value = httprequest('POST', body, '/Abak/Transact/GetGroupedTransacts', headers=headers)
    transactions_to_clean = output_value.get('data')
    projects_clean = get_projects(None, "", 'python')
    project_map = {}
    contexts = get_contexts()
    for project in projects_clean:
        project_cons = project['Display'][0:60]
        for context in contexts:
            if project['Id'] == contexts[context]['Project']:
                project_map[project_cons] = context
                continue

    if context_filter:
        if context_filter not in [context for context in contexts]:
            raise click.exceptions.BadOptionUsage('context_filter', f"Context '{context_filter}' doesn't exist!")
        transactions = [
            transaction 
            for transaction in transactions_to_clean 
            if context_filter == project_map.get(transaction.get('ProjectName')[0:60])
        ]
    else: 
        transactions = transactions_to_clean

    if output == 'json':
        print(json.dumps(transactions))
    elif output in ['table', 'wide']:


        if show_totals:
            date_datetime = datetime.strptime(date, config['date_format']) 
            click.echo("For the " + (f"month of {calendar.month_name[date_datetime.month]}" if query_range == "Monthly" else "day of " + date if query_range == "Daily" else "week of " + date) + 
                        ", here are the totals:")
            totals = {} 
            for row in transactions:
                totals[row['ProjectName']] = totals.get(row['ProjectName'], 0) + row.get('Quantity', 0)
            totals['TOTAL'] = sum([ totals[total] for total in totals])
            headers = ['Context', 'Quanty', 'Hour Price']
            to_print = []
            total_price = 0
            for total in totals:
                if total != "TOTAL":
                    to_print.append([project_map.get(total[0:60], "Context Unassigned"), totals[total], str(contexts.get(project_map.get(total[0:60]), {'Price': 0}).get('Price', 0))])
                    total_price += totals.get(total) * contexts.get(project_map.get(total[0:60]), {'Price': 0}).get('Price', 0)
                else:
                    to_print.append([total, totals[total], str(total_price)])
            print(tabulate(to_print, headers=headers, numalign="left"))
        else:
            output_format = {
                "Weekday": "Date",
                "Date": "Date",
                "Project": "ProjectName",
                "Description": "Description",
                "Hrs": "Quantity"
            }
            if output == 'wide':
                output_format['Date'] = 'Date'
                output_format['ID'] = "Id"

            if show_id:
                output_format['ID'] = "Id"
            headers = [header for header in output_format]
            rows = []
            for row in transactions:
                instance = []
                for header in headers:
                    if header == 'Weekday':
                        date_text = row[output_format.get(header)].split('T00')[0]
                        date_instance = datetime.strptime(date_text, config['date_format'])
                        instance.append(date_instance.strftime('%A'))
                    elif header == 'Project':
                        instance.append(project_map.get(row[output_format[header]][0:60], row[output_format[header]]))
                    else:
                        instance.append(row[output_format.get(header)] if header != "Date" else row[output_format.get(header)].split('T00')[0])
                rows.append(instance)
            print(tabulate(rows, headers=headers))
    elif output == "python":
        return transactions

def convert_date(text):
    start = text.find('new Date')
    finish = text.find('),', start) + 1
    next = finish
    while next > 0:
        date_to_convert = text[start:finish]
        date_to_convert_array = date_to_convert[date_to_convert.find('(')+1:date_to_convert.find(')')].split(',')
        date_output = f'"{date_to_convert_array[0]}-{int(date_to_convert_array[1])+1:02d}-{date_to_convert_array[2]}T00:00:00"'
        text = text.replace(date_to_convert, date_output)
        start = text.find('new Date', next)
        finish = text.find('),', start) + 1
        next = finish

    return text

@click.command(name='set')
@click.option('--date', help="Date to set the timesheet entry", callback=validate_entry_date, show_default="Today")
@click.option('--yesterday', help="Sets the date to yesterday", is_flag=True)
@click.option('--description', '-d', help="Description of the activities for that day", required=False)
@click.option('--context', help="Context to use when setting the timesheet", default=lambda: environ.get('current_context', None), show_default="selected context - 'abak context select'")
@click.option('--hours', '-h', help="Number of work-hours to be assigned for the timesheet entry.", default=8.0, type=float)
@click.option('--client-id', '-c', help="ID of the client to assign the timesheet entry.", default=lambda: environ.get('client_id', None), show_default="selected client_id")
@click.option('--project-id', '-p', help="ID of the project to to assign the timesheet entry.", default=lambda: environ.get('project_id', None), show_default="selected project_id")
@click.option('--bs', help="For when you need to dazzle!", is_flag=True)
@click.pass_context
def timesheet_set(ctx, date, description, context, hours, client_id, project_id, bs, yesterday):
    '''
    Creates a timesheet entry in ABAK
    '''
    days_of_the_week = ["MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY", "SUNDAY"]
    config = get_config()
    if yesterday:
        date = datetime.strftime(datetime.now() - timedelta(days=1), format=config['date_format'])
    elif date: 
        if date.upper() in days_of_the_week:
            date = datetime.strftime(datetime.now() - timedelta(days=datetime.now().weekday()) + timedelta(days=days_of_the_week.index(date.upper())), format=config['date_format'])
    elif not date:
        date = datetime.strftime(datetime.now(), format=config['date_format'])

    if context:
        contexts = get_contexts()
        selected_context = contexts.get(context, None)
        client_id = selected_context['Client']
        project_id = selected_context['Project']
    elif context != None:
        raise Sorry(f"context \"{context}\" was not found")

    if bs:
        description = generate_bs()
        click.confirm(f"Are you sure you would like to add '{description}' to your timesheet?", abort=True)
    else:
        validate_description(ctx, 'description', description)

    set_timesheet_entry(client_id, project_id, date, description, hours)

@click.command(name='apply')
@click.pass_context
@click.option('--file', '-f', type=click.Path(exists=True, readable=True), help="File to read the timesheet entries from")
@click.option('--example', type=click.Choice(['json', 'yaml', 'yml']), help="Outputs an example file")
def timesheet_apply(ctx, file, example):
    '''
    Applies all the entries from the given file
    Currently supported formats:
        json, yaml and yml
    '''
    config = get_config()
    if example:
        first_day = datetime.today() - timedelta(days=datetime.today().weekday())
        example_object = {
            'clients': [
                {
                    'clientId': client_id,
                    'projects': [
                        {
                            'projectId': project_id,
                            'entries': [
                                {
                                    'date': datetime.strftime(date_entry, format=config['date_format']),
                                    'hours': 8,
                                    'description': "Something Meaningful"
                                }
                            for date_entry in [first_day + timedelta(days=x) for x in range(2)]]
                        }
                    for project_id in ['project_id_1', 'project_id_2']]
                }
            for client_id in ['client_id_1', 'client_id_2']]
        }
        if example == 'json':
            click.echo(json.dumps(example_object))
        elif example in ['yaml', 'yml']:
            click.echo(yaml.safe_dump(example_object))
        exit()

    if not file:
        click.exceptions.MissingParameter(param=click.Parameter(['--file', '-f']))

    supported_formats = ['json', 'yaml', 'yml']
    format = file.split('.')[-1]

    if format not in supported_formats:
        raise Sorry("files of the format '." + format + "' are not supported")
    
    with open(file, 'r') as filereader:
        if format == "json":
            data = json.loads(filereader.read())
        elif format in ["yaml", 'yml']:
            data = yaml.load(filereader.read(), Loader=yaml.BaseLoader)


    for client in data.get('clients'):
        for project in client.get('projects'):
            for entry in project.get('entries'):
                validate_entry_date(None, None, entry.get('date'))
                validate_description(None, None, entry.get('description'))
                set_timesheet_entry(client.get('clientId'), project.get('projectId'), entry.get('date'), entry.get('description'), entry.get('hours'))

def set_timesheet_entry(client_id, project_id, date, description, hours):
    option_not_none('client id', client_id)
    option_not_none('project id', project_id)
    config = get_config()
    date = datetime.strftime(datetime.strptime(date, config['date_format']), get_date_format_from_abak(config['abak_date_format']))
    headers = {
        "Content-Type": "application/x-www-form-urlencoded"
    }
    body = {
        "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "fieldDate": date,
        "typeTab": "timesheetDetail",
        "": "true",
        "rangeField": "Weekly",
        "defaultDate": date,
        "fieldId": "",
        "fieldIsDuplicatingTs": False,
        "fieldEmployeeId": config['user_id'],
        "isPhaseEmplAssigned": "",
        "isDefaultTaskAssignedOnEmp": "",
        "isDefaultPayCodeAssignedOnEmp": "",
        "defaultPayCodeAssignedOnEmp": "",
        "taskDefaultPayCode": "",
        "isDefaultTaskAssignedOnFun": "",
        "isDefaultPayCodeAssignedOnFun": "",
        "defaultPayCodeAssignedOnFun": "",
        "isPhaseFunctionAssigned": "",
        "taskDefaultTaskCode": "",
        "phaseDefaultPayCode": "",
        "functionDefaultTaskCode": "",
        "phaseDefaultDepartment": "",
        "phaseDefaultReference": "",
        "isPhaseBillable": "",
        "phaseDefaultExpenseType": "",
        "defaultDepartmentAssignedOnEmp": "",
        "defaultReferenceAssignedOnEmp": "",
        "defaultExpenseTypeAssignedOnEmp": "",
        "defaultDepartmentAssignedOnFun": "",
        "defaultReferenceAssignedOnFun": "",
        "defaultExpenseTypeAssignedOnFun": "",
        "defaultTaskAssignedOnEmp": "",
        "defaultTaskAssignedOnFun": "",
        "defaultPayCodeOnTaskAssignedOnEmp": "",
        "defaultPayCodeOnTaskAssignedOnFun": "",
        "phaseDescFr": "",
        "phaseDescEn": "",
        "phaseDesc": "",
        "queryTextField": "Consulting by Associate (CONS)",
        "fieldManualCost": "",
        "fieldManualDayCost": "",
        "fieldManualSelling": "",
        "fieldManualDaySelling": "",
        "fieldcheckPaySystem": "fieldcheckPaySystem",
        "fieldcheckPaySystemSate": "fieldcheckPaySystemSate",
        "payCodeIsInList": "true",
        "clientIsInList": "true",
        "projectIsInList": "true",
        "taskCodeIsInList": "true",
        "departmentIsInList": "true",
        "datebookId": "",
        "fieldClientId_Value": client_id,
        "fieldClientId_SelIndex": -1,
        "fieldProjectId_Value": project_id,
        "fieldProjectId_SelIndex": -1,
        "fieldTaskCode_Value": "CONS",
        "fieldTaskCode_SelIndex": -1,
        "fieldPhaseId": "",
        "fieldPhaseId_current": "",
        "fieldPhaseIdReal": "",
        "fieldMSProjectCode": "",
        "fieldMsProject": "",
        "fieldDescription_Value": description,
        "fieldDescription": description,
        "fieldDescription_SelIndex": "-1",
        "fieldReference": "",
        "fieldIsPrintOnInvoice": "fieldIsPrintOnInvoice",
        "fieldDepartmentId_Value": "",
        "fieldDepartmentId": "",
        "fieldDepartmentId_SelIndex": -1,
        "fieldPayCode_Value": "REG",
        "fieldPayCode": "Regular (REG)",
        "fieldPayCode_SelIndex": -1,
        "fieldIsAffectingTimebank": "fieldIsAffectingTimebank",
        "fieldQuantity": str(hours) + " hrs",
        "fieldTimeStart_Value": "",
        "fieldTimeStart": "",
        "fieldTimeStart_SelIndex": -1,
        "fieldTimeEnd_Value": "",
        "fieldTimeEnd": "",
        "fieldTimeEnd_SelIndex": -1,
        "fieldLunchTime": "0.00 hrs",
        "fieldIsBillable": "fieldIsBillable",
        "fieldBillableQuantity": str(hours) + " hrs",
        "panelGroupInvoicableTimesheetDetailFieldUnitCost": "0.0000 $",
        "TimesheetDetailFieldSubTotal": "0.00 $",
        "fieldEmployeeTimesheetDetail": "",
        "fieldNote": "",
        "billableDescriptionTimesheetDetail": "",
        "fieldBillableNoteTimesheetDetail": "",
        "tabPanel_ActiveTab": {"tabDetail":0}
    }

    timesheet_entry = httprequest('POST', body, "/Abak/Timesheet/Edit", headers=headers)
    if not timesheet_entry.get('success'):
        raise Sorry(timesheet_entry.get('extraParams', {'Message': 'there was an error with your request'}).get('Message'))
    click.echo("Timesheet entry " + timesheet_entry.get('extraParams', {"newID": ""}).get('newID') + " created successully!")


def get_weekly_timesheet(ctx, *args, **kwargs):
    config = get_config()
    date = datetime.strftime(datetime.now(), format=config['date_format'])
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
    }
    body = {
        "groupBy": "TransactType",
        "groupDir": "ASC",
        "summaryFields": "Quantity",
        "summaryTypes": "sum",
        "sort": "Date",
        "dir": "ASC",
        "employe": config['user_id'],
        "date": date + "T00:00:00",
        "range": range
    }
    output_value = httprequest('POST', body, '/Abak/Transact/GetGroupedTransacts', headers=headers)
    return [(row['Id'], row['Description']) for row in output_value.get('data')]

@click.command(name='delete')
@click.pass_context
@click.argument('id')
def timesheet_delete(ctx, id):
    '''
    Deletes a timesheet entry from ABAK
    '''
    headers = {
        "Content-Type": "application/json; charset=utf-8"
    }
    body = {
        "transacts":[
            {
                "key":id,
                "value":"T"
            }
        ]
    }

    result = httprequest('POST', body, "/Abak/Transact/DeleteTransacts", is_json=True, headers=headers)
    click.echo("Timesheet entry " + id + " deleted successfully!")


@click.command(name='approve')
@click.pass_context
@click.option('--start-date', '-s', help="Start date of the range for timesheet approval", callback=validate_entry_date, required=True)
@click.option('--end-date', '-e', help="End date of the range for timesheet approval", callback=validate_entry_date, required=True)
@click.option('--remove', is_flag=True, help="Unapproves the timesheet range")
def timesheet_approve(ctx, start_date, end_date, remove):
    '''
    Approve timesheet entries
    '''
    config = get_config() 
    body = {
        "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "employeeId": config['user_id'],
        "groupBy": "Date",
        "groupDir": "ASC",
        "summaryFields": "Quantity",
        "summaryFields": "BillableQuantity",
        "summaryFields": "TotalExpense",
        "summaryTypes": "sum",
        "summaryTypes": "sum",
        "summaryTypes": "sum",
        "sort": "Date",
        "dir": "DESC",
        "startDate": start_date + "T00:00:00",
        "endDate": end_date + "T00:00:00",
        "approvalType": "Timesheet",
        "start": "0",
        "limit": "50"
    }

    output_value = httprequest('POST', body, "/Abak/Approval/GetApprovalsList", is_json=True)

    output_format = {
        "Date": "Date",
        "Project": "ProjectName",
        "Description": "Description",
        "Hrs": "Quantity"
    }
    headers = [header for header in output_format]
    rows = []
    for row in output_value['data']:
        instance = []
        for header in headers:
            instance.append(row[output_format.get(header)] if header != "Date" else row[output_format.get(header)].split('T00')[0])
        rows.append(instance)
    if not remove:
        click.echo("Here are the timesheet entries to be approved:")
    else:
        click.echo("Here are the timesheet entries to be Unapproved:")

    print(tabulate(rows, headers=headers))
    if not remove:
        click.confirm("Are you sure that you want to approve these timesheets from " + start_date + " to " + end_date + "?", abort=True)
    else:
        click.confirm("Are you sure that you want to remove the approval for these timesheets from " + start_date + " to " + end_date + "?", abort=True)

    if not remove:
        path = "/Abak/Approval/ApproveRangeFromApprobation"
    else:
        path = "/Abak/Approval/UnapproveRange"
    body = {
        "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "approvalType": "Timesheet",
        "employeeId": config['user_id'],
        "startDate": start_date,
        "endDate": end_date
    }

    result = httprequest('POST', body, path, is_json=True)
    if not remove:
        click.echo("Timesheets approved successfully!")
    else:
        click.echo("Timesheets unapproved successfully!")



timesheet.add_command(timesheet_list)
timesheet.add_command(timesheet_set)
timesheet.add_command(timesheet_delete)
timesheet.add_command(timesheet_approve)
timesheet.add_command(timesheet_apply)



import webbrowser
import click
import timesheets
import client
import project
import abak_config
import os
import keyring
from abak_context import context

from requests.exceptions import ConnectionError

from abak_shared_functions import (
    write_config_file,
    get_config,
    authenticate,
    httprequest,
    Sorry
)


@click.group()
@click.pass_context
def abak(ctx):
    """
    Abak UI, NEVER AGAIN!
    """
    config = get_config()
    if ctx.invoked_subcommand not in ["login", "config"]:
        try:
            result = httprequest("get", None, "/Abak/Transact/GetEmployee_Optimized")
            config["user_id"] = result["data"][0]["Id"]
            write_config_file(config["config_file_path"], config)
            [
                os.environ.setdefault(key, config[key])
                for key in config
                if key
                not in [
                    "app_dir",
                    "config_file_path",
                    "authenticated",
                    "headers",
                    "token",
                    "contexts"
                ]
            ]
        except ConnectionError:
            Sorry(
                "it seems that you are not connected to the internet or the endpoint is not available"
            )
        except Exception:
            try:
                authenticate(
                    config["username"],
                    keyring.get_password("fancy-abak", config["username"]),
                    config["endpoint"],
                )
            except KeyError:
                click.echo("Please login first!")
                exit(127)
            except Exception as exception:
                raise exception


def get_password(ctx, param, value):
    if not value:
        return keyring.get_password("fancy-abak", ctx.params['username'])
    else:
        return value

@click.command()
@click.option(
    "-u", "--username", help="the username to use for login", required=True, prompt=True
)
@click.option(
    "-p", "--password", help="the password to use for login", prompt=True, hide_input=True, prompt_required=False, callback=get_password
)
@click.option(
    "-e",
    "--endpoint",
    help="The endpoint where abak is hosted",
    required=True,
    prompt=True,
)
def login(username, password, endpoint):
    authenticate(username, password, endpoint)
    click.echo("Login Successul!")


@click.command(name="open")
def open_browser():
    config = get_config()
    webbrowser.open(config["endpoint"])



abak.add_command(login)
abak.add_command(open_browser)

abak.add_command(timesheets.timesheet)
abak.add_command(client.client)
abak.add_command(project.project)
abak.add_command(abak_config.config)
abak.add_command(context)


if __name__ == "__main__":
    abak()


import click
import json
import yaml
from abak_shared_functions import get_clean_config

@click.command(name="show")
@click.option('-o', '--output', help="Type of output of the configuration", default='json', type=click.Choice(['json', 'yaml']))
@click.argument('key', required=False)
@click.pass_context
def show_config(ctx, key, output):
    '''
    Shows the current configuration file for JIRA communication and default values

    Args:
        key (str): the key to show
    '''
    configuration = get_clean_config()
    if not key:
        if output == 'json':
            click.echo(json.dumps(configuration))
        elif output == 'yaml':
            click.echo(yaml.dump(configuration))
    elif configuration.get(key):
        click.echo(configuration[key])



import click
from abak_shared_functions import write_config_file, get_config

def get_keys(ctx, args, incomplete):
    config = get_config()
    return [key for key in config.keys() if incomplete in key]

@click.command(name='remove')
@click.option('--key', '-k', help="The key to be removed", required=True, prompt=True, shell_complete=get_keys)
@click.pass_context
def remove_config(ctx, key):
    '''
    Removes a key from the configuration file

    Args:
        key (str): The key to be removed
    '''
    remove_configuration_key(key)

def remove_configuration_key(key):
    configuration = get_config()
    if configuration.get(key):
        del configuration[key]
    write_config_file(configuration['config_file_path'], configuration)



import requests
from .exceptions import Sorry
from .abak_configuration_functions import get_config
import json
import re

def convert_date(text):
    start = text.find('new Date')
    finish = text.find('),', start) + 1
    next_instance = finish
    while next_instance > 0:
        date_to_convert = text[start:finish]
        date_to_convert_array = date_to_convert[date_to_convert.find('(')+1:date_to_convert.find(')')].split(',')
        date_output = f'"{date_to_convert_array[0]}-{int(date_to_convert_array[1])+1:02d}-{date_to_convert_array[2]}T00:00:00"'
        text = text.replace(date_to_convert, date_output)
        start = text.find('new Date', next_instance)
        finish = text.find('),', start) + 1
        next_instance = finish

    return text

def httprequest(request_type, body, path, is_json=False, headers={}):
    request_function = {
        'GET': requests.get,
        'POST': requests.post 
    }.get(request_type.upper(), None)

    config = get_config()
    headers["Cookie"] = config['token']

    if is_json:
        result = request_function(config['endpoint'] + path, headers=headers, json=body)
    else:
        result = request_function(config['endpoint'] + path, headers=headers, data=body)
    try:
        result.raise_for_status()
        output_value = json.loads(convert_date(result.text))
        return output_value
    except requests.exceptions.HTTPError as error:
        raise Sorry(re.findall('(<title>)(.*)(</title>)', result.text)[0][1])   


import click
import os
import json
import requests
import re

def get_headers(config):
    return {}

def get_config():
    app_name = '.abakctl'
    config_file = 'config.json'
    app_dir = click.get_app_dir(app_name)
    config_file_path = os.path.join(app_dir, config_file)
    config = read_configfile(config_file_path)
    config['authenticated'] = True
    config['headers'] = get_headers(config)
    config['app_dir'] = app_dir
    config['config_file_path'] = config_file_path
    return config

def read_configfile(config_file_path: str) -> dict:
    '''
    This function reads the contents of the config file for jiractl and dumps it as a dict for later use

    Args:
        config_file_path (string): path of the config file

    Returns:
        dict: dictionary with the configuration to run jiractl
    '''
    if not os.path.isfile(config_file_path):
        create_default_configfile()
    with open(config_file_path, 'r') as file_reader:
        config_dict = json.loads(file_reader.read())
    return config_dict

def create_default_configfile():

    app_name = '.abakctl'
    config_file = 'config.json'
    app_dir = click.get_app_dir(app_name)
    dir_split = app_dir.split('/')
    for i in range(2, len(dir_split)+1, 1):
        current_path = "/".join(dir_split[:i])
        print(current_path)
        if not os.path.isdir(current_path):
                os.mkdir(current_path)
    config = {}
    write_config_file(os.path.join(app_dir, config_file), config)

def write_config_file(file_path, data):
    if data.get('headers'): del data['headers']
    if data.get('app_dir'): del data['app_dir']
    if data.get('config_file_path'): del data['config_file_path']
    if data.get('authenticated'): del data['authenticated']
    with open(file_path, 'w') as file_writer:
        file_writer.write(json.dumps(data))

def authenticate(username, password, endpoint):
    config = get_config()
    body = {
        'username': username,
        'password': password,
        'device': 'W'
    }
    headers = {
        "Content-Type": "application/x-www-form-urlencoded"
    }
    result = requests.post(endpoint + '/Abak/Account/Authenticate', data=body, headers=headers)
    result.raise_for_status()
    if result.headers['Set-Cookie'].find('AbakUsername') > 0:
        config["token"] = result.headers['Set-Cookie']
        config['endpoint'] = endpoint
        config['username'] = username
        config['abak_date_format'] = re.findall(r'(?:\{)(.*?)(?:\})', re.findall(r'(?:AbakDateFormat=)(.*?)(?:;)', result.headers['Set-Cookie'])[0])[-1]
        if not config.get('date_format'):
            config['date_format'] = "%Y-%m-%d"
        write_config_file(config['config_file_path'], config)
    else:
        click.echo(result.json()['errorMessage'])
        exit(10)

def get_clean_config():
    configuration = get_config()
    configuration['token'] = "**********"
    del configuration['headers']
    return configuration


def get_date_format_from_abak(abak_date):
    format_date = {
        'M/dd/yy': '%m/%d/%y',
        "d/M/y": '%m/%d/%y',
        "dd/MM/yy": "%d/%m/%y",
        "dd/MM/yyyy": "%d/%m/%Y",
        "d/M/yy": "%d/%m/%y",
        "d/M/yyyy": "%d/%m/%Y",
        "d/MM/yyyy": "%d/%m/%Y",
        "dd/M/yyyy": "%d/%m/%Y",
        "dd-MM-yyyy": '%d-%m-%Y', 
        "d-MM-yyyy": '%d-%m-%Y', 
        "d-M-yyyy": '%d-%m-%Y', 
        "d-M-yy": '%d-%m-%y', 
        "dd-M-yyyy": '%d-%m-%Y',
        "d-M-y":  '%d-%m-%y',
        "M-dd-yy": '%m-%d-%y',
        "Y/M/d": '%Y/%m/%d',
        "yyyy/MM/dd": '%Y/%m/%d',
        "Y-M-d": '%Y-%m-%d', 
        "yyyy-MM-dd": '%Y-%m-%d',
        "yyyy/M/d": '%Y/%m/%d',
        "yyyy-M-d": '%Y-%m-%d',
        "y/M/d": '%y/%m/%d',
        "yy/MM/dd": '%y/%m/%d',
        "y-M-d": '%y-%m-%d',
        "yy-MM-dd": '%y-%m-%d'
    }

    return format_date.get(abak_date)


from abak_config.remove import remove_configuration_key
from abak_config.set import set_configuration_key
import click
import json
from iterfzf import iterfzf
from tabulate import tabulate
from abak_shared_functions import get_config, httprequest

@click.group()
@click.pass_context
def client(ctx):
    '''
    Find clients to assign timesheet entries
    '''
    pass


@click.command(name='list')
@click.pass_context
@click.option('-o', '--output', help="The format of the output of this command", type=click.Choice(['json', 'table']), default='table')
@click.option('--query-text', '-q', help="Text to search for in the client name", default="")
def client_list(ctx, output, query_text):
    '''
    Lists the clients available
    '''
    get_clients(query_text, output)

@click.command(name='select')
@click.pass_context
def client_select(ctx):
    '''
    Selects the default client ID to use
    '''
    clients_list = get_clients("", 'python')
    if len(clients_list) > 1:
        selected_dirty = iterfzf([" - ".join([item.get('Id'), item.get('DisplayName')]) for item in clients_list])
        selected = selected_dirty.split(' - ')[0]
    else:
        selected = clients_list[0].get('Id')
    set_configuration_key(selected, 'client_id')
    remove_configuration_key('project_id')
    click.echo("Client " + selected + " selected as default!")

def get_clients(query_text, output):
    config = get_config()
    body = {
        "queryText": query_text,
        "start": 0,
        "limit": 22,
        "employeeId": config['user_id']
    }

    clients = httprequest('GET', body, "/Abak/Common/GetTimesheetClientsPaginated")

    if output == "table": 
        headers = ["Id", "DisplayName"]
        table = tabulate([[client[key]for key in client if key in headers]for client in clients.get('data')], headers=headers)
        print(table)
    elif output == 'json':
        print(json.dumps(clients.get('data')))
    elif output == 'python':
        return clients.get('data')


client.add_command(client_list)
client.add_command(client_select)


from setuptools import setup, find_packages

setup(
    name='fancy-abak',
    version='1.1.0',
    author='Gui Martins',
    url='https://fancywhale.ca/',
    author_email='gmartins@fancywhale.ca',
    packages=find_packages(),
    include_package_data=True,
    py_modules=['abak'],
    python_requires='>=3.6',
    install_requires=[
        "certifi==2022.12.7",
        "charset-normalizer==3.0.1",
        "click==8.1.3",
        "idna==3.4",
        "importlib-metadata==6.0.0",
        "iterfzf==0.5.0.20.0",
        "jaraco.classes==3.2.3",
        "keyring==23.13.1",
        "more-itertools==9.0.0",
        "pyaml==21.10.1",
        "PyYAML==6.0",
        "requests==2.28.2",
        "tabulate==0.9.0",
        "urllib3==1.26.14",
        "zipp==3.12.0",
    ],
    entry_points='''
        [console_scripts]
        abak=abak:abak
    ''',
    long_description="CLI Tool to interface with Abak"
)



import click
from .remove import remove_config
from .set import set_config
from .show import show_config


@click.group()
@click.pass_context
def config(ctx):
    '''
    Group of commands to manage the jiractl command line
    '''
    pass


config.add_command(show_config)
config.add_command(set_config)
config.add_command(remove_config)





from random import choice

def generate_bs():
    verbs = ['aggregate', 'architect', 'benchmark', 'brand', 'cultivate', 'deliver', 'deploy', 'disintermediate', 'drive', 'e-enable', 'embrace', 'empower', 'enable', 'engage', 'engineer', 'enhance', 'envision', 'envisioneer', 'evolve', 'expedite', 'exploit', 'extend', 'facilitate', 'generate', 'grow', 'harness', 'implement', 'incentivize', 'incubate', 'innovate', 'integrate', 'impact', 'iterate', 'leverage', 'matrix', 'maximize', 'mesh', 'monetize', 'morph', 'optimize', 'orchestrate', 'productize', 'recontextualize', 'redefine', 'reintermediate', 'reinvent', 'repurpose', 'revolutionize', 'scale', 'seize', 'strategize', 'streamline', 'syndicate', 'synergize', 'synthesize', 'target', 'transform', 'transition', 'unleash', 'utilize', 'visualize', 'whiteboard']

    adjectives = ['24/365', '24/7', 'B2B', 'B2C', 'back-end', 'best-of-breed', 'bleeding-edge', 'blockchain', 'bricks-and-clicks', 'clicks-and-mortar', 'cloud', 'collaborative', 'compelling', 'cross-platform', 'cross-media', 'customized', 'cutting-edge', 'distributed', 'dot-com', 'down stream', 'dynamic', 'e-business', 'efficient', 'end-to-end', 'enterprise', 'extensible', 'frictionless', 'front-end', 'global', 'granular', 'holistic', 'impactful', 'innovative', 'integrated', 'interactive', 'intuitive', 'killer', 'leading-edge', 'magnetic', 'mission-critical', 'next-generation', 'one-to-one', 'open-source', 'out-of-the-box', 'plug-and-play', 'proactive', 'real-time', 'revolutionary', 'rich', 'robust', 'scalable', 'seamless', 'sexy', 'sticky', 'strategic', 'synergistic', 'transparent', 'turn-key', 'ubiquitous', 'user-centric', 'upstream', 'value-added', 'venture', 'vertical', 'viral', 'virtual', 'visionary', 'web-enabled', 'wireless', 'world-class']

    nouns = ['action-items', 'ai', 'applications', 'architectures', 'bandwidth', 'channels', 'communities', 'content', 'convergence', 'dark web', 'deliverables', 'e-business', 'e-commerce', 'e-markets', 'e-services', 'e-tailers', 'experiences', 'eyeballs', 'functionalities', 'infomediaries', 'infrastructures', 'initiatives', 'interfaces', 'machine learning', 'markets', 'methodologies', 'metrics', 'mindshare', 'models', 'networks', 'niches', 'paradigms', 'partnerships', 'platforms', 'portals', 'relationships', 'ROI', 'SaaS', 'start ups', 'synergies', 'web-readiness', 'schemas', 'solutions', 'supply-chains', 'systems', 'technologies', 'users', 'vortals', 'web services']

    verb, adjective, noun = 'bullshit', 'bullshit', 'bullshit'


    while noun in adjective or verb in adjective or adjective in noun or verb in noun or adjective in verb or noun in verb:
        verb = choice(verbs)
        adjective = choice(adjectives)
        noun = choice(nouns)
    
        
    bullshit = verb + ' ' + adjective + ' ' + noun
    
    return bullshit

if __name__ == '__main__':
    print(generate_bs())